%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per Öberg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per Öberg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

function psSimResult=psConvertSimOutput(psSimOutput,psSimStruct,resampMethod)
%
% psSimResult=psConvertSimOutput(psSimOutput,psSimStruct,caOffset)
%
% Interpolates and renames data according to simulation.    
% Arguments:
% psSimOutput    -  data generated from simulation och psModel 

   
  nanIndx = find(~isnan(psSimOutput.pcb)); % Use pcb since that't the one
                                           % wer'e using 
  theNames= fieldnames(psSimOutput);
  for ii = 1:length(theNames)
      psSimOutput.(theNames{ii}) = psSimOutput.(theNames{ii})(nanIndx,:);
      psSimResult.rawData.(theNames{ii}) = psSimOutput.(theNames{ii});
  end
  
  psSimResult.rawData.thetaCyl = psSimOutput.theta;        
  psSimResult.rawData.t = psSimResult.rawData.time;
  % Calculate common pressure by takig mean value of doublets
  %   prepare ...
  psSimResult.rawData.p= psSimOutput.pcb;
  psSimResult.rawData.thetaCorrected=psSimResult.rawData.theta-psSimStruct.psMeas.caOffset;
  psSimResult.rawData.tCorrected=psSimResult.rawData.time-psSimStruct.psMeas.caOffset/psSimStruct.psSimPar.w;
  interpolVarNames =...
      {'t','tCorrected','thetaCorrected','theta','p','Tb','Tu','Vb','Vu'};
 
  % Interpolate rawData 
  simdatanames = interpolVarNames;
  if nargin <= 2
      resampMethod = 'cubic';
  end
  % Always start at a fraction of a degree that fits with the resampleFac
  % Use theta as sample base
  thetaDiff=2*pi/psSimStruct.psSimPar.resampleFac;
  thetaStart=psSimStruct.psSimPar.thetaStart-...
      mod(psSimStruct.psSimPar.thetaStart,thetaDiff)+thetaDiff; 
  thetaStop=psSimStruct.psSimPar.thetaStop-...
      mod(psSimStruct.psSimPar.thetaStop,thetaDiff); 
  samplePoints = (thetaStart:thetaDiff:thetaStop)';

  theta = psSimResult.rawData.theta;
  correctedTheta = psSimResult.rawData.thetaCorrected;
  for ii = 1:length(simdatanames)
      if psSimStruct.psSimPar.giveWarnings
          disp(sprintf('Interpolating data from: %s',simdatanames{ii}))
      end
      rawData = psSimResult.rawData.(simdatanames{ii});
      resampData=interp1(theta,rawData,samplePoints,resampMethod,'extrap');
      psSimResult.resampData.(simdatanames{ii}) = resampData;
      resampData=interp1(correctedTheta,rawData,samplePoints,resampMethod,'extrap');
      psSimResult.correctedResampData.(simdatanames{ii}) = resampData;
  end
  
  psSimResult.correctedResampData.varInfo = {'This data has been reampled';...
                      'using the psMeas.caOffset value as offset.';...
                      '';...
                      'The correctedTheta is the variable that';...
                      'ends on the correct bounds in this data'};
  psSimResult.psSimStruct = psSimStruct;
  
  
if 0 
  figure
  plot(psSimResult.rawData.theta,psSimResult.rawData.p)
  hold on
  plot(psSimResult.rawData.thetaCorrected,psSimResult.rawData.p,'--')
  grid on
  plot(psSimResult.resampData.theta,psSimResult.resampData.p,'r:')
  plot(psSimResult.correctedResampData.thetaCorrected,psSimResult.correctedResampData.p,'r:')
end